@@ -49,9 +49,6 @@ class AgentJinjaTemplater(object):
              'base_path': CONF.haproxy_amphora.base_path,
              'bind_host': CONF.haproxy_amphora.bind_host,
              'bind_port': CONF.haproxy_amphora.bind_port,
-             # TODO(nmagnezi): user_group is Deprecated in Pike,
-             # should be Removed in R cycle.
-             'user_group': CONF.haproxy_amphora.user_group,
              'controller_list': CONF.health_manager.controller_ip_port_list,
              'debug': CONF.debug,
              'haproxy_cmd': CONF.haproxy_amphora.haproxy_cmd,
@@ -59,4 +56,6 @@ class AgentJinjaTemplater(object):
              'heartbeat_key': CONF.health_manager.heartbeat_key,
              'use_upstart': CONF.haproxy_amphora.use_upstart,
              'respawn_count': CONF.haproxy_amphora.respawn_count,
-             'respawn_interval': CONF.haproxy_amphora.respawn_interval})
+             'respawn_interval': CONF.haproxy_amphora.respawn_interval,
+             'amphora_udp_driver': CONF.amphora_agent.amphora_udp_driver,
+             'topology': CONF.controller_worker.loadbalancer_topology})
