@@ -140,6 +140,8 @@ class ListenersController(base.BaseController):
             try:
                 self.cert_manager.set_acls(context, ref)
                 self.cert_manager.get_cert(context, ref, check_only=True)
+            except exceptions.UnreadablePKCS12:
+                raise
             except Exception:
                 bad_refs.append(ref)
 
@@ -426,7 +428,8 @@ class ListenersController(base.BaseController):
             LOG.info("Sending delete Listener %s to provider %s", id,
                      driver.name)
             provider_listener = (
-                driver_utils.db_listener_to_provider_listener(db_listener))
+                driver_utils.db_listener_to_provider_listener(
+                    db_listener, for_delete=True))
             driver_utils.call_provider(driver.name, driver.listener_delete,
                                        provider_listener)
 
