@@ -13,11 +13,10 @@
 # under the License.
 #
 
-import logging
-
 from cryptography import fernet
 from oslo_config import cfg
 from oslo_db import exception as odb_exceptions
+from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import uuidutils
 import six
@@ -878,7 +877,8 @@ class UpdateAmphoraInfo(BaseDatabaseTask):
         self.amphora_repo.update(
             db_apis.get_session(), amphora_id,
             lb_network_ip=compute_obj.lb_network_ip,
-            cached_zone=compute_obj.cached_zone)
+            cached_zone=compute_obj.cached_zone,
+            image_id=compute_obj.image_id)
         return self.amphora_repo.get(db_apis.get_session(), id=amphora_id)
 
 
@@ -1542,9 +1542,9 @@ class GetAmphoraeFromLoadbalancer(BaseDatabaseTask):
         """
         amphorae = []
         for amp in loadbalancer.amphorae:
-            a = self.amphora_repo.get(db_apis.get_session(), id=amp.id)
-
-            if a is None or a.status == constants.DELETED:
+            a = self.amphora_repo.get(db_apis.get_session(), id=amp.id,
+                                      show_deleted=False)
+            if a is None:
                 continue
             amphorae.append(a)
         return amphorae
