@@ -130,6 +130,8 @@ class SessionPersistence(base_models.BASE):
             name="fk_session_persistence_session_persistence_type_name"),
         nullable=False)
     cookie_name = sa.Column(sa.String(255), nullable=True)
+    persistence_timeout = sa.Column(sa.Integer(), nullable=True)
+    persistence_granularity = sa.Column(sa.String(64), nullable=True)
     pool = orm.relationship("Pool", uselist=False,
                             back_populates="session_persistence")
 
@@ -188,6 +190,7 @@ class Member(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
     ip_address = sa.Column('ip_address', sa.String(64), nullable=False)
     protocol_port = sa.Column(sa.Integer, nullable=False)
     weight = sa.Column(sa.Integer, nullable=True)
+    backup = sa.Column(sa.Boolean(), nullable=False)
     monitor_address = sa.Column(sa.String(64), nullable=True)
     monitor_port = sa.Column(sa.Integer, nullable=True)
     provisioning_status = sa.Column(
@@ -345,6 +348,7 @@ class LoadBalancer(base_models.BASE, base_models.IdMixin,
     amphorae = orm.relationship("Amphora", uselist=True,
                                 back_populates="load_balancer")
     server_group_id = sa.Column(sa.String(36), nullable=True)
+    provider = sa.Column(sa.String(64), nullable=True)
     vip = orm.relationship('Vip', cascade='delete', uselist=False,
                            backref=orm.backref('load_balancer', uselist=False))
     pools = orm.relationship('Pool', cascade='delete', uselist=True,
@@ -393,6 +397,7 @@ class Vip(base_models.BASE):
     subnet_id = sa.Column(sa.String(36), nullable=True)
     network_id = sa.Column(sa.String(36), nullable=True)
     qos_policy_id = sa.Column(sa.String(36), nullable=True)
+    octavia_owned = sa.Column(sa.Boolean(), nullable=True)
 
 
 class Listener(base_models.BASE, base_models.IdMixin,
@@ -452,6 +457,10 @@ class Listener(base_models.BASE, base_models.IdMixin,
 
     peer_port = sa.Column(sa.Integer(), nullable=True)
     insert_headers = sa.Column(sa.PickleType())
+    timeout_client_data = sa.Column(sa.Integer, nullable=True)
+    timeout_member_connect = sa.Column(sa.Integer, nullable=True)
+    timeout_member_data = sa.Column(sa.Integer, nullable=True)
+    timeout_tcp_inspect = sa.Column(sa.Integer, nullable=True)
 
     # This property should be a unique list of the default_pool and anything
     # referenced by enabled L7Policies with at least one rule that also
@@ -491,7 +500,7 @@ class SNI(base_models.BASE):
     position = sa.Column(sa.Integer(), nullable=True)
 
 
-class Amphora(base_models.BASE, base_models.IdMixin):
+class Amphora(base_models.BASE, base_models.IdMixin, models.TimestampMixin):
 
     __data_model__ = data_models.Amphora
 
@@ -525,6 +534,7 @@ class Amphora(base_models.BASE, base_models.IdMixin):
     vrrp_id = sa.Column(sa.Integer(), nullable=True)
     vrrp_priority = sa.Column(sa.Integer(), nullable=True)
     cached_zone = sa.Column(sa.String(255), nullable=True)
+    image_id = sa.Column(sa.String(36), nullable=True)
     load_balancer = orm.relationship("LoadBalancer", uselist=False,
                                      back_populates='amphorae')
 
