@@ -13,6 +13,7 @@
 
 import itertools
 
+from octavia.policies import amphora
 from octavia.policies import base
 from octavia.policies import healthmonitor
 from octavia.policies import l7policy
@@ -21,6 +22,7 @@ from octavia.policies import listener
 from octavia.policies import loadbalancer
 from octavia.policies import member
 from octavia.policies import pool
+from octavia.policies import provider
 from octavia.policies import quota
 
 
@@ -34,5 +36,7 @@ def list_rules():
         loadbalancer.list_rules(),
         member.list_rules(),
         pool.list_rules(),
+        provider.list_rules(),
         quota.list_rules(),
+        amphora.list_rules(),
     )
