@@ -24,6 +24,7 @@ from oslo_utils import uuidutils
 from octavia.common import clients
 from octavia.common import constants
 from octavia.common import data_models
+from octavia.common import exceptions
 from octavia.network import base as network_base
 from octavia.network import data_models as network_models
 from octavia.network.drivers.neutron import allowed_address_pairs
@@ -200,6 +201,49 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertRaises(network_base.DeallocateVIPException,
                           self.driver.deallocate_vip, vip)
 
+    def test_deallocate_vip_when_secgrp_has_allocated_ports(self):
+        max_retries = 1
+        conf = oslo_fixture.Config(cfg.CONF)
+        conf.config(group="networking", max_retries=max_retries)
+
+        lb = dmh.generate_load_balancer_tree()
+        lb.vip.load_balancer = lb
+        vip = lb.vip
+        show_port = self.driver.neutron_client.show_port
+        show_port.return_value = {'port': {
+            'device_owner': allowed_address_pairs.OCTAVIA_OWNER}}
+        delete_port = self.driver.neutron_client.delete_port
+        list_ports = self.driver.neutron_client.list_ports
+        list_security_groups = self.driver.neutron_client.list_security_groups
+        delete_sec_grp = self.driver.neutron_client.delete_security_group
+        security_groups = {
+            'security_groups': [
+                {'id': t_constants.MOCK_SECURITY_GROUP_ID}
+            ]
+        }
+        list_security_groups.return_value = security_groups
+        delete_grp_results = [
+            network_base.DeallocateVIPException
+            for _ in range(max_retries + 1)]  # Total tries = max_retries + 1
+        delete_grp_results.append(None)
+        delete_sec_grp.side_effect = delete_grp_results
+        list_ports.side_effect = [{
+            "ports": [t_constants.MOCK_NEUTRON_PORT['port'],
+                      t_constants.MOCK_NEUTRON_PORT2['port']]}]
+        self.driver.deallocate_vip(vip)
+        # First we expect the amp's ports to be deleted
+        dp_calls = [mock.call(amp.vrrp_port_id) for amp in lb.amphorae]
+        # Then after the SG delete fails, extra hanging-on ports are removed
+        dp_calls.append(mock.call(t_constants.MOCK_PORT_ID))
+        # Lastly we remove the vip port
+        dp_calls.append(mock.call(vip.port_id))
+        self.assertEqual(len(dp_calls), delete_port.call_count)
+        delete_port.assert_has_calls(dp_calls)
+        dsg_calls = [mock.call(t_constants.MOCK_SECURITY_GROUP_ID)
+                     for _ in range(max_retries + 2)]  # Max fail + one success
+        self.assertEqual(len(dsg_calls), delete_sec_grp.call_count)
+        delete_sec_grp.assert_has_calls(dsg_calls)
+
     def test_deallocate_vip_when_port_not_found(self):
         lb = dmh.generate_load_balancer_tree()
         vip = data_models.Vip(port_id='1')
@@ -208,6 +252,17 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         show_port.side_effect = neutron_exceptions.PortNotFoundClient
         self.driver.deallocate_vip(vip)
 
+    def test_deallocate_vip_when_port_not_found_for_update(self):
+        lb = dmh.generate_load_balancer_tree()
+        vip = data_models.Vip(port_id='1')
+        vip.load_balancer = lb
+        show_port = self.driver.neutron_client.show_port
+        show_port.return_value = {'port': {
+            'device_owner': allowed_address_pairs.OCTAVIA_OWNER}}
+        update_port = self.driver.neutron_client.update_port
+        update_port.side_effect = neutron_exceptions.PortNotFoundClient
+        self.driver.deallocate_vip(vip)
+
     def test_deallocate_vip_when_port_not_owned_by_octavia(self):
         lb = dmh.generate_load_balancer_tree()
         lb.vip.load_balancer = lb
@@ -325,6 +380,43 @@ class TestAllowedAddressPairsDriver(base.TestCase):
                                         t_constants.MOCK_VRRP_IP2])
             self.assertEqual(lb.vip.ip_address, amp.ha_ip)
 
+    @mock.patch('octavia.network.drivers.neutron.utils.'
+                'convert_port_dict_to_model')
+    def test_plug_vip_amp__port_create_fails(self, mock_convert):
+        lb = dmh.generate_load_balancer_tree()
+
+        subnet = network_models.Subnet(id=t_constants.MOCK_VIP_SUBNET_ID,
+                                       network_id=t_constants.MOCK_VIP_NET_ID)
+
+        list_ports = self.driver.neutron_client.list_ports
+        port1 = t_constants.MOCK_MANAGEMENT_PORT1['port']
+        port2 = t_constants.MOCK_MANAGEMENT_PORT2['port']
+        list_ports.side_effect = [{'ports': [port1]}, {'ports': [port2]}]
+        port_create = self.driver.neutron_client.create_port
+        port_create.side_effect = [Exception('Create failure')]
+        self.assertRaises(network_base.PlugVIPException,
+                          self.driver._plug_amphora_vip,
+                          lb.amphorae[0], subnet)
+        mock_convert.assert_not_called()
+        self.driver.neutron_client.delete_port.assert_not_called()
+
+    def test_plug_vip_amp_port_attach_fails(self):
+        lb = dmh.generate_load_balancer_tree()
+
+        subnet = network_models.Subnet(id=t_constants.MOCK_VIP_SUBNET_ID,
+                                       network_id=t_constants.MOCK_VIP_NET_ID)
+
+        list_ports = self.driver.neutron_client.list_ports
+        port1 = t_constants.MOCK_MANAGEMENT_PORT1['port']
+        port2 = t_constants.MOCK_MANAGEMENT_PORT2['port']
+        list_ports.side_effect = [{'ports': [port1]}, {'ports': [port2]}]
+        network_attach = self.driver.nova_client.servers.interface_attach
+        network_attach.side_effect = [Exception('Attach failure')]
+        self.assertRaises(network_base.PlugVIPException,
+                          self.driver._plug_amphora_vip,
+                          lb.amphorae[0], subnet)
+        self.driver.neutron_client.delete_port.assert_called_once()
+
     def _set_safely(self, obj, name, value):
         if isinstance(obj, dict):
             current = obj.get(name)
@@ -404,8 +496,10 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertRaises(network_base.AllocateVIPException,
                           self.driver.allocate_vip, fake_lb)
 
-    def test_allocate_vip_when_no_port_provided(self):
-        port_create_dict = copy.copy(t_constants.MOCK_NEUTRON_PORT)
+    @mock.patch('octavia.network.drivers.neutron.base.BaseNeutronDriver.'
+                '_check_extension_enabled', return_value=True)
+    def test_allocate_vip_when_no_port_provided(self, mock_check_ext):
+        port_create_dict = copy.deepcopy(t_constants.MOCK_NEUTRON_PORT)
         port_create_dict['port']['device_owner'] = (
             allowed_address_pairs.OCTAVIA_OWNER)
         port_create_dict['port']['device_id'] = 'lb-1'
@@ -439,6 +533,43 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
         self.assertEqual(fake_lb.id, vip.load_balancer_id)
 
+    @mock.patch('octavia.network.drivers.neutron.base.BaseNeutronDriver.'
+                '_check_extension_enabled', return_value=False)
+    def test_allocate_vip_when_no_port_provided_tenant(self, mock_check_ext):
+        port_create_dict = copy.deepcopy(t_constants.MOCK_NEUTRON_PORT)
+        port_create_dict['port']['device_owner'] = (
+            allowed_address_pairs.OCTAVIA_OWNER)
+        port_create_dict['port']['device_id'] = 'lb-1'
+        create_port = self.driver.neutron_client.create_port
+        create_port.return_value = port_create_dict
+        show_subnet = self.driver.neutron_client.show_subnet
+        show_subnet.return_value = {'subnet': {
+            'id': t_constants.MOCK_SUBNET_ID,
+            'network_id': t_constants.MOCK_NETWORK_ID
+        }}
+        fake_lb_vip = data_models.Vip(subnet_id=t_constants.MOCK_SUBNET_ID,
+                                      network_id=t_constants.MOCK_NETWORK_ID)
+        fake_lb = data_models.LoadBalancer(id='1', vip=fake_lb_vip,
+                                           project_id='test-project')
+        vip = self.driver.allocate_vip(fake_lb)
+        exp_create_port_call = {
+            'port': {
+                'name': 'octavia-lb-1',
+                'network_id': t_constants.MOCK_NETWORK_ID,
+                'device_id': 'lb-1',
+                'device_owner': allowed_address_pairs.OCTAVIA_OWNER,
+                'admin_state_up': False,
+                'tenant_id': 'test-project',
+                'fixed_ips': [{'subnet_id': t_constants.MOCK_SUBNET_ID}]
+            }
+        }
+        create_port.assert_called_once_with(exp_create_port_call)
+        self.assertIsInstance(vip, data_models.Vip)
+        self.assertEqual(t_constants.MOCK_IP_ADDRESS, vip.ip_address)
+        self.assertEqual(t_constants.MOCK_SUBNET_ID, vip.subnet_id)
+        self.assertEqual(t_constants.MOCK_PORT_ID, vip.port_id)
+        self.assertEqual(fake_lb.id, vip.load_balancer_id)
+
     def test_unplug_vip_errors_when_update_port_cant_find_port(self):
         lb = dmh.generate_load_balancer_tree()
         list_ports = self.driver.neutron_client.list_ports
@@ -584,8 +715,12 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             server=t_constants.MOCK_COMPUTE_ID, port_id=port2.get('id'))
 
     def test_update_vip(self):
-        listeners = [data_models.Listener(protocol_port=80, peer_port=1024),
-                     data_models.Listener(protocol_port=443, peer_port=1025)]
+        listeners = [data_models.Listener(protocol_port=80, peer_port=1024,
+                                          protocol=constants.PROTOCOL_TCP),
+                     data_models.Listener(protocol_port=443, peer_port=1025,
+                                          protocol=constants.PROTOCOL_TCP),
+                     data_models.Listener(protocol_port=50, peer_port=1026,
+                                          protocol=constants.PROTOCOL_UDP)]
         vip = data_models.Vip(ip_address='10.0.0.2')
         lb = data_models.LoadBalancer(id='1', listeners=listeners, vip=vip)
         list_sec_grps = self.driver.neutron_client.list_security_groups
@@ -606,17 +741,27 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             'security_group_rule': {
                 'security_group_id': 'secgrp-1',
                 'direction': 'ingress',
-                'protocol': 'TCP',
+                'protocol': 'tcp',
                 'port_range_min': 1024,
                 'port_range_max': 1024,
                 'ethertype': 'IPv4'
             }
         }
+        expected_create_rule_udp_peer = {
+            'security_group_rule': {
+                'security_group_id': 'secgrp-1',
+                'direction': 'ingress',
+                'protocol': 'tcp',
+                'port_range_min': 1026,
+                'port_range_max': 1026,
+                'ethertype': 'IPv4'
+            }
+        }
         expected_create_rule_2 = {
             'security_group_rule': {
                 'security_group_id': 'secgrp-1',
                 'direction': 'ingress',
-                'protocol': 'TCP',
+                'protocol': 'tcp',
                 'port_range_min': 1025,
                 'port_range_max': 1025,
                 'ethertype': 'IPv4'
@@ -626,20 +771,39 @@ class TestAllowedAddressPairsDriver(base.TestCase):
             'security_group_rule': {
                 'security_group_id': 'secgrp-1',
                 'direction': 'ingress',
-                'protocol': 'TCP',
+                'protocol': 'tcp',
                 'port_range_min': 443,
                 'port_range_max': 443,
                 'ethertype': 'IPv4'
             }
         }
+        expected_create_rule_udp = {
+            'security_group_rule': {
+                'security_group_id': 'secgrp-1',
+                'direction': 'ingress',
+                'protocol': 'udp',
+                'port_range_min': 50,
+                'port_range_max': 50,
+                'ethertype': 'IPv4'
+            }
+        }
+
         create_rule.assert_has_calls([mock.call(expected_create_rule_1),
+                                      mock.call(expected_create_rule_udp_peer),
                                       mock.call(expected_create_rule_2),
-                                      mock.call(expected_create_rule_3)])
+                                      mock.call(expected_create_rule_3),
+                                      mock.call(expected_create_rule_udp)],
+                                     any_order=True)
 
     def test_update_vip_when_listener_deleted(self):
-        listeners = [data_models.Listener(protocol_port=80),
+        listeners = [data_models.Listener(protocol_port=80,
+                                          protocol=constants.PROTOCOL_TCP),
                      data_models.Listener(
                          protocol_port=443,
+                         protocol=constants.PROTOCOL_TCP,
+                         provisioning_status=constants.PENDING_DELETE),
+                     data_models.Listener(
+                         protocol_port=50, protocol=constants.PROTOCOL_UDP,
                          provisioning_status=constants.PENDING_DELETE)]
         vip = data_models.Vip(ip_address='10.0.0.2')
         lb = data_models.LoadBalancer(id='1', listeners=listeners, vip=vip)
@@ -648,7 +812,8 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         fake_rules = {
             'security_group_rules': [
                 {'id': 'rule-80', 'port_range_max': 80, 'protocol': 'tcp'},
-                {'id': 'rule-22', 'port_range_max': 443, 'protocol': 'tcp'}
+                {'id': 'rule-22', 'port_range_max': 443, 'protocol': 'tcp'},
+                {'id': 'rule-udp-50', 'port_range_max': 50, 'protocol': 'tcp'}
             ]
         }
         list_rules = self.driver.neutron_client.list_security_group_rules
@@ -656,7 +821,8 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         delete_rule = self.driver.neutron_client.delete_security_group_rule
         create_rule = self.driver.neutron_client.create_security_group_rule
         self.driver.update_vip(lb)
-        delete_rule.assert_called_once_with('rule-22')
+        delete_rule.assert_has_calls(
+            [mock.call('rule-22'), mock.call('rule-udp-50')])
         self.assertTrue(create_rule.called)
 
     def test_update_vip_when_no_listeners(self):
@@ -696,6 +862,28 @@ class TestAllowedAddressPairsDriver(base.TestCase):
         self.driver.update_vip(lb)
         delete_rule.assert_called_once_with('ssh-rule')
 
+    def test_update_vip_when_security_group_missing(self):
+        listeners = []
+        vip = data_models.Vip(ip_address='10.0.0.2')
+        lb = data_models.LoadBalancer(id='1', listeners=listeners, vip=vip)
+        list_sec_grps = self.driver.neutron_client.list_security_groups
+        list_sec_grps.return_value = {'security_groups': []}
+        self.assertRaises(exceptions.MissingVIPSecurityGroup,
+                          self.driver.update_vip,
+                          lb)
+
+    @mock.patch('octavia.network.drivers.neutron.allowed_address_pairs.'
+                'AllowedAddressPairsDriver._update_security_group_rules')
+    def test_update_vip_for_delete_when_security_group_missing(self,
+                                                               update_rules):
+        listeners = []
+        vip = data_models.Vip(ip_address='10.0.0.2')
+        lb = data_models.LoadBalancer(id='1', listeners=listeners, vip=vip)
+        list_sec_grps = self.driver.neutron_client.list_security_groups
+        list_sec_grps.return_value = {'security_groups': []}
+        self.driver.update_vip(lb, for_delete=True)
+        update_rules.assert_not_called()
+
     def test_failover_preparation(self):
         original_dns_integration_state = self.driver.dns_integration_enabled
         self.driver.dns_integration_enabled = False
